@@ -54,6 +54,8 @@
 			;; match data.
 			(save-excursion
 			  (set-buffer buf)
+			  (if vm-fsfemacs-mule-p
+			      (set-buffer-multibyte nil))
 			  (widen)
 			  (erase-buffer)
 			  (insert header)
@@ -126,6 +128,10 @@ The saved messages are flagged as `filed'."
 		 (setq auto-folder (vm-auto-select-folder
 				    vm-message-pointer
 				    vm-auto-folder-alist))
+		 ;; Don't let user archive into the same folder
+		 ;; that they are visiting.
+		 (not (eq (vm-get-file-buffer auto-folder)
+			  (current-buffer)))
 		 (or (null arg)
 		     (y-or-n-p
 		      (format "Save message %s in folder %s? "
@@ -487,7 +493,10 @@ Output, if any, is displayed.  The message is not altered."
 	      ((equal prefix-arg '(64))
 	       (narrow-to-region (vm-vheaders-of m) (vm-text-end-of m)))
 	      (t (narrow-to-region (point) (vm-text-end-of m))))
-	(let ((pop-up-windows (and pop-up-windows (eq vm-mutable-windows t))))
+	(let ((pop-up-windows (and pop-up-windows (eq vm-mutable-windows t)))
+	      ;; call-process-region calls write-region.
+	      ;; don't let it do CR -> LF translation.
+	      (selective-display nil))
 	  (call-process-region (point-min) (point-max)
 			       (or shell-file-name "sh")
 			       nil buffer nil shell-command-switch command)))
@@ -553,7 +562,10 @@ Output, if any, is displayed.  The message is not altered."
 			(vm-mime-external-content-types-alist nil))
 		    (vm-decode-mime-layout (vm-mm-layout m)))
 		  (let ((pop-up-windows (and pop-up-windows
-					     (eq vm-mutable-windows t))))
+					     (eq vm-mutable-windows t)))
+			;; call-process-region calls write-region.
+			;; don't let it do CR -> LF translation.
+			(selective-display nil))
 		    (if need-tempfile
 			(write-region (point-min) (point-max)
 				      tempfile nil 0))
@@ -568,7 +580,10 @@ Output, if any, is displayed.  The message is not altered."
 	  (widen)
 	  (narrow-to-region (vm-vheaders-of m) (vm-text-end-of m))
 	  (let ((pop-up-windows (and pop-up-windows
-				     (eq vm-mutable-windows t))))
+				     (eq vm-mutable-windows t)))
+		;; call-process-region calls write-region.
+		;; don't let it do CR -> LF translation.
+		(selective-display nil))
 	    (if need-tempfile
 		(write-region (point-min) (point-max)
 			      tempfile nil 0))
